/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package org.glassfish.scripting.jruby.monitor;

import org.glassfish.external.probe.provider.annotations.ProbeListener;
import org.glassfish.external.probe.provider.annotations.ProbeParam;
import org.glassfish.external.statistics.StringStatistic;
import org.glassfish.external.statistics.annotations.Reset;
import org.glassfish.external.statistics.impl.StringStatisticImpl;
import org.glassfish.gmbal.AMXMetadata;
import org.glassfish.gmbal.Description;
import org.glassfish.gmbal.ManagedAttribute;
import org.glassfish.gmbal.ManagedObject;

import java.util.logging.Logger;

/**
 * @author Vivek Pandey
 */
@AMXMetadata(type="jruby-application-mon", group="monitoring")
@ManagedObject
@Description("JRuby Application Statistics")
public class JRubyApplicationStatsProvider {

    private final Logger logger = Logger.getLogger(JRubyApplicationStatsProvider.class.getName());
    private final StringStatisticImpl appName = new StringStatisticImpl("ApplicationName", null, "Ruby application name");
    private final StringStatisticImpl framework = new StringStatisticImpl("RubyApplicationFramework", null, "Ruby application Framework");
    private final StringStatisticImpl environment = new StringStatisticImpl("ApplicationEnvironment", null, "Ruby application environment");
    private final StringStatisticImpl jrubyVersion = new StringStatisticImpl("JRubyVersion", null, "JRuby version");
    private final StringStatisticImpl contextPath = new StringStatisticImpl("ContextPath", null, "Context path of Ruby application");


    public JRubyApplicationStatsProvider(String appName, String contextPath) {
        this.contextPath.setCurrent(contextPath);
        this.appName.setCurrent(appName);
    }

    @ProbeListener("glassfish:jruby:jruby-container:jrubyModuleStartedEvent")
    public void jrubyModuleStartedEvent(
            @ProbeParam("appName") String appName,
            @ProbeParam("applicationType") String applicationType,
            @ProbeParam("environment") String environment,
            @ProbeParam("jrubyVersion") String jrubyVersion
            ){
        //appName can't be null
        if(appName == null){
            logger.warning("appName is null in jrubyModuleStartedEvent! Looks like the event is not triggered properly, ignoring this monitoring event.");
        }else if(this.appName.getCurrent().equals(appName)){
            logger.fine("JRubyApplicationStatsProvider:jrubyModuleStartedEvent invoked");
            this.appName.setCurrent(appName);
            this.framework.setCurrent(applicationType);
            this.environment.setCurrent(environment);
            this.jrubyVersion.setCurrent(jrubyVersion);
        }
    }

    @ProbeListener("glassfish:jruby:jruby-container:jrubyModuleStoppedEvent")
    public void jrubyModuleStoppedEvent(
            @ProbeParam("appName") String appName
            ){
        if(appName == null){
            logger.warning("appName is null in jrubyModuleStoppedEvent! Looks like the event is not triggered properly, ignoring this monitoring event.");
            return;
        }
        logger.fine("JRubyApplicationStatsProvider:jrubyModuleStoppedEvent invoked");
        if(appName.equals(this.appName.getCurrent())){
            this.appName.reset();
            this.framework.reset();
            this.environment.reset();
            this.jrubyVersion.reset();
            this.contextPath.reset();
        }
    }

    @ManagedAttribute(id="appname")
    @Description("Ruby application name")
    public StringStatistic getAppName(){
        return appName;
    }

    @ManagedAttribute(id = "contextpath")
    @Description("Application context-path")
    public StringStatistic getContextPath() {
        return contextPath;
    }
    

    @ManagedAttribute(id="rubyframework")
    public  StringStatistic getRubyFramework(){
        return framework;
    }

    @ManagedAttribute(id="environment")
    public  StringStatistic getEnvironment(){
        return environment;
    }

    @ManagedAttribute(id="jrubyversion")
    public  StringStatistic getJrubyVersion(){
        return jrubyVersion;
    }


    /**
     * v3 monitoring by default tries to reset all the Statistic when level changes from OFF->HIGH.
     * We dont want this so have a noop operation.
     */
    @Reset
    public void reset(){
        //noop.
    }

}
