/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package org.glassfish.scripting.jruby.monitor;

import org.glassfish.external.probe.provider.annotations.ProbeListener;
import org.glassfish.external.probe.provider.annotations.ProbeParam;
import org.glassfish.external.statistics.CountStatistic;
import org.glassfish.external.statistics.StringStatistic;
import org.glassfish.external.statistics.annotations.Reset;
import org.glassfish.external.statistics.impl.CountStatisticImpl;
import org.glassfish.external.statistics.impl.StringStatisticImpl;
import org.glassfish.gmbal.AMXMetadata;
import org.glassfish.gmbal.Description;
import org.glassfish.gmbal.ManagedAttribute;
import org.glassfish.gmbal.ManagedObject;
import org.glassfish.scripting.monitor.utils.TimeStat;

import java.util.HashMap;
import java.util.logging.Logger;


/**
 * The stats provider per application. This means It should be created by JRubyApplication during deployment.
 *
 * @author Vivek Pandey
 */
@AMXMetadata(type="jruby-http-mon", group="monitoring")
@ManagedObject
@Description("Ruby application HTTP stats")
public class JRubyHttpStatsProvider {

    private final Logger logger = Logger.getLogger(JRubyHttpStatsProvider.class.getName());

    private final CountStatisticImpl averageProcessingTime = new CountStatisticImpl("AverageRequestProcessingTime", CountStatisticImpl.UNIT_MILLISECOND, "Average request processing time");
    private final CountStatisticImpl reqPerSec = new CountStatisticImpl("Requests/Seconds", CountStatisticImpl.UNIT_COUNT, "Requests per seconds");

    private final CountStatisticImpl requestCount = new CountStatisticImpl("RequestCounter", CountStatisticImpl.UNIT_COUNT, "Number of HTTP requests received");
    private final CountStatisticImpl errorCount = new CountStatisticImpl("ErrorCount", CountStatisticImpl.UNIT_COUNT, "Number of responses with a status code greater than 400");
    private final CountStatisticImpl error200Count = new CountStatisticImpl("Count200", CountStatisticImpl.UNIT_COUNT, "Number of responses with a status code equal to 200");
    private final CountStatisticImpl error2xxCount = new CountStatisticImpl("Count2xx", CountStatisticImpl.UNIT_COUNT, "Number of responses with a status code in the 2xx range");
    private final CountStatisticImpl error302Count = new CountStatisticImpl("Count302", CountStatisticImpl.UNIT_COUNT, "Number of responses with a status code equal to 302");
    private final CountStatisticImpl error304Count = new CountStatisticImpl("Count304", CountStatisticImpl.UNIT_COUNT, "Number of responses with a status code equal to 304");
    private final CountStatisticImpl error3xxCount = new CountStatisticImpl("Count3xx", CountStatisticImpl.UNIT_COUNT, "Number of responses with a status code in the 3xx range");
    private final CountStatisticImpl error400Count = new CountStatisticImpl("Count400", CountStatisticImpl.UNIT_COUNT, "Number of responses with a status code equal to 400");
    private final CountStatisticImpl error401Count = new CountStatisticImpl("Count401", CountStatisticImpl.UNIT_COUNT, "Number of responses with a status code equal to 401");
    private final CountStatisticImpl error403Count = new CountStatisticImpl("Count403", CountStatisticImpl.UNIT_COUNT, "Number of responses with a status code equal to 403");
    private final CountStatisticImpl error404Count = new CountStatisticImpl("Count404", CountStatisticImpl.UNIT_COUNT, "Number of responses with a status code equal to 404");
    private final CountStatisticImpl error4xxCount = new CountStatisticImpl("Count4xx", CountStatisticImpl.UNIT_COUNT, "Number of responses with a status code in the 4xx range");
    private final CountStatisticImpl error503Count = new CountStatisticImpl("Count503", CountStatisticImpl.UNIT_COUNT, "Number of responses with a status code equal to 503");
    private final CountStatisticImpl error5xxCount = new CountStatisticImpl("Count5xx", CountStatisticImpl.UNIT_COUNT, "Number of responses with a status code in the 5xx range");
    private final CountStatisticImpl errorOtherCount = new CountStatisticImpl("CountOther", CountStatisticImpl.UNIT_COUNT, "Number of responses with other status codes");


    private final HashMap<String, CountStatisticImpl> errorMap = new HashMap<String, CountStatisticImpl>();
    private TimeStat timer;


    private final StringStatisticImpl appName = new StringStatisticImpl("ApplicationName", null, "Ruby application name");
    private final StringStatisticImpl contextPath = new StringStatisticImpl("ContextPath", null, "Context path of Ruby application");
    private StringStatisticImpl serverAddress = new StringStatisticImpl("ServerAddress", null, "Server address");

    public JRubyHttpStatsProvider(String appName, String contextPath) {
        this.appName.setCurrent(appName);
        this.contextPath.setCurrent(contextPath);
        this.errorMap.put("200", error200Count);
        this.errorMap.put("2", error2xxCount);
        this.errorMap.put("302", error302Count);
        this.errorMap.put("304", error304Count);
        this.errorMap.put("3", error3xxCount);
        this.errorMap.put("400", error400Count);
        this.errorMap.put("401", error401Count);
        this.errorMap.put("403", error403Count);
        this.errorMap.put("404", error404Count);
        this.errorMap.put("4", error4xxCount);
        this.errorMap.put("503", error503Count);
        this.errorMap.put("5", error5xxCount);
        this.errorMap.put("9", errorOtherCount);
        this.timer = new TimeStat();
    }


    @ProbeListener("glassfish:jruby:http:requestStartEvent")
    public void requestStartEvent(
            @ProbeParam("appName") String appName,
            @ProbeParam("contextPath") String contextPath,
            @ProbeParam("serverName") String serverName,
            @ProbeParam("port") int port
    ) {
        if(appName == null){
            logger.warning("appName is null in requestStartEvent! Looks like the event is not triggered properly, ignoring this monitoring event.");
        }else if(this.appName.getCurrent().equals(appName) && this.contextPath.getCurrent().equals(contextPath)){
            serverAddress.setCurrent(serverName+":"+port);
            requestCount.increment();
            timer.entry();
        }
    }

    @ProbeListener("glassfish:jruby:http:requestEndEvent")
    public void requestEndEvent(
            @ProbeParam("appName") String appName,
            @ProbeParam("contextPath") String contextPath,
            @ProbeParam("statusCode") int statusCode
    ) {
        if(appName == null){
            logger.warning("appName is null in requestEndEvent! Looks like the event is not triggered properly, ignoring this monitoring event.");
        }else if(this.appName.getCurrent().equals(appName) && this.contextPath.getCurrent().equals(contextPath)){
            timer.exit();
            incrementStatsCounter(statusCode);
        }
    }

    @ManagedAttribute(id = "applicationname")
    @Description("Application name")
    public StringStatistic getApplicationName() {
        return appName;
    }


    @ManagedAttribute(id = "contextpath")
    @Description("Application context-path")
    public StringStatistic getContextPath() {
        return contextPath;
    }


    @ManagedAttribute
    @Description("Server address")
    public StringStatistic getAddress() {
        return serverAddress;
    }

    @ManagedAttribute
    @Description("Total requests processed so far")
    public CountStatistic getRequestCount() {
        return requestCount;
    }

    @ManagedAttribute
    @Description("Requests/Second")
    public CountStatistic getRequestPerSecond() {
        long totalTime = timer.getTotalTime(TimeStat.UNIT.SECOND);
        long rps = ((totalTime == 0)?0:(requestCount.getCount() / totalTime));
        reqPerSec.setCount(rps);
        return reqPerSec;
    }

    @ManagedAttribute
    @Description("Average request processing time in milliseconds")
    public CountStatistic getAverageProcessingTime() {
        if (requestCount.getCount() == 0)
            averageProcessingTime.setCount(0);
        else
            averageProcessingTime.setCount(timer.getTotalTime(TimeStat.UNIT.MILLISECOND) / requestCount.getCount());
        return averageProcessingTime;
    }


    @ManagedAttribute(id = "errorcount")
    @Description("")
    public CountStatistic getErrorCount() {
        return errorCount;
    }

    @ManagedAttribute(id = "count200")
    @Description("")
    public CountStatistic getError200Count() {
        return error200Count;
    }

    @ManagedAttribute(id = "count2xx")
    @Description("")
    public CountStatistic getError2xxCount() {
        return error2xxCount;
    }

    @ManagedAttribute(id = "count302")
    @Description("")
    public CountStatistic getError302Count() {
        return error302Count;
    }

    @ManagedAttribute(id = "count304")
    @Description("")
    public CountStatistic getError304Count() {
        return error304Count;
    }

    @ManagedAttribute(id = "count3xx")
    @Description("")
    public CountStatistic getError3xxCount() {
        return error3xxCount;
    }

    @ManagedAttribute(id = "count400")
    @Description("")
    public CountStatistic getError400Count() {
        return error400Count;
    }

    @ManagedAttribute(id = "count401")
    @Description("")
    public CountStatistic getError401Count() {
        return error401Count;
    }

    @ManagedAttribute(id = "count403")
    @Description("")
    public CountStatistic getError403Count() {
        return error403Count;
    }

    @ManagedAttribute(id = "count404")
    @Description("")
    public CountStatistic getError404Count() {
        return error404Count;
    }

    @ManagedAttribute(id = "count4xx")
    @Description("")
    public CountStatistic getError4xxCount() {
        return error4xxCount;
    }

    @ManagedAttribute(id = "count503")
    @Description("")
    public CountStatistic getError503Count() {
        return error503Count;
    }

    @ManagedAttribute(id = "count5xx")
    @Description("")
    public CountStatistic getError5xxCount() {
        return this.error5xxCount;
    }

    @ManagedAttribute(id = "countother")
    @Description("")
    public CountStatistic getErrorOtherCount() {
        return this.errorOtherCount;
    }

    private void incrementStatsCounter(int statusCode) {
        if (statusCode >= 400)
            errorCount.increment();
        
        CountStatisticImpl errorCounter = errorMap.get(Integer.toString(statusCode));
        if(errorCounter != null){
            errorCounter.increment();

            //also increment the Nxx errors, for example 2xx will include 200 status as well.
            errorCounter = errorMap.get(Integer.toString(Math.round(statusCode / 100)));
            if(errorCounter != null){
                errorCounter.increment();
            }
            return;
        }

        errorCounter = errorMap.get(Integer.toString(Math.round(statusCode / 100)));
        if (errorCounter == null){
            errorCounter = errorMap.get("9");
        }
        errorCounter.increment();
    }

    @Reset
    public void reset(){
        averageProcessingTime.reset();
        reqPerSec.reset();
        requestCount.reset();
        errorCount.reset();
        error200Count.reset();
        error2xxCount.reset();
        error302Count.reset();
        error304Count.reset();
        error3xxCount.reset();
        error400Count.reset();
        error401Count.reset();
        error403Count.reset();
        error404Count.reset();
        error4xxCount.reset();
        error503Count.reset();
        error5xxCount.reset();
        errorOtherCount.reset();
    }



}
